class AR_Bitmap
 {
  protected:

   float x,y;
   float scale_x,scale_y;
   float angle,center_x,center_y;
   int life;
   unsigned char alpha;
   bool enable_tint_r,enable_tint_g,enable_tint_b;
   int flip_horizontal,flip_vertical;
   bool enable;
   AR_Display     *display;
   ALLEGRO_BITMAP *bitmap;

   void checkValues()
    {
	 if(angle < 0)   angle = 360 + angle;
	 if(angle > 360) angle = angle - 360;
	}

   void reset()
    {
	 // Reset variables to default
     x        = y        = 0;
     scale_x  = scale_y  = 1.0;
     center_x = center_y = 0;
     angle           = 0;
     alpha           = 255;
     life            = 0;
     flip_horizontal = 0;
     flip_vertical   = 0;
     enable          = true;
     enable_tint_r = enable_tint_g = enable_tint_b = true;
	}

  public:

   AR_Bitmap(string filename, AR_Display *display, AR_Bitmap *p = NULL)
    {
     reset();
     setDisplay(display);

     // Share or not share data with other AR_Bitmap objects
     if(!p) loadBitmap(filename);
     else   bitmap = p->bitmap;
    }

   ~AR_Bitmap() { al_destroy_bitmap(bitmap); }

   void loadBitmap(string filename)
    {
	 bitmap = al_load_bitmap(filename.c_str());
	 if(!bitmap) { AR_Util::errorBox(display->getDisplay(),string("File '") + string(filename) + string("' not found!")); exit(1); }
    }

   void draw()
    {
     if(!enable) return;

     checkValues();
	 al_draw_tinted_scaled_rotated_bitmap(bitmap,
	                                      al_map_rgba(alpha * enable_tint_r,
	                                                  alpha * enable_tint_g,
	                                                  alpha * enable_tint_b,
	                                                  alpha),
	                                      center_x,center_y,
	                                      x,y,
	                                      scale_x,scale_y,
	                                      AR_Util::radians(angle),
	                                      flip_horizontal | flip_vertical);
    }

   void enableFlipHorizontal() { flip_horizontal = ALLEGRO_FLIP_HORIZONTAL; }
   
   void enableFlipVertical()   { flip_vertical   = ALLEGRO_FLIP_VERTICAL; }

   void disableFlipHorizontal() { flip_horizontal = 0; }
   
   void disableFlipVertical()   { flip_vertical   = 0; }

   bool isEnableTintR() { return enable_tint_r; }

   bool isEnableTintG() { return enable_tint_g; }

   bool isEnableTintB() { return enable_tint_b; }

   void setX(float x)                 { this->x = x;               }

   void setY(float y)                 { this->y = y;               }

   void setScaleX(float scale_x)      { this->scale_x = scale_x;   }

   void setScaleY(float scale_y)      { this->scale_y = scale_y;   }

   void setCenterX(float center_x)    { this->center_x = center_x; }

   void setCenterY(float center_y)    { this->center_y = center_y; }

   void setAngle(float angle)         { this->angle = angle;       }

   void setAlpha(unsigned char alpha) { this->alpha = alpha;       }

   void setLife(int life)             { this->life = life;         }

   void setEnable(bool enable)        { this->enable = enable;     }

   void setEnableTintRGB(bool r, bool g, bool b)
    {
	 enable_tint_r = r;
	 enable_tint_g = g;
	 enable_tint_b = b;
    }

   void setDisplay(AR_Display *display) { this->display = display; }

   float getX()             { return x;       }

   float getY()             { return y;       }

   float *getXP()           { return &x;      }

   float *getYP()           { return &y;      }

   float getScaleX()        { return scale_x; }

   float getScaleY()        { return scale_y; }

   float getCenterX()       { return center_x; }

   float getCenterY()       { return center_y; }

   float getAngle()         { return angle;   }

   int getWidth()           { return al_get_bitmap_width(bitmap);  }

   int getHeight()          { return al_get_bitmap_height(bitmap); }

   int getLife()            { return life;    }

   unsigned char getAlpha() { return alpha;   }

   bool getEnable()         { return enable;  }

   AR_Display      *getDisplay() { return display; }

   ALLEGRO_BITMAP  *getBitmap()  { return bitmap; }

 };
