class AR_ParticleGenerator
 {
  private:
   
   bool can_generate;
   bool random_enable;
   float x,y;
   float start_scale_x,start_scale_y;
   float acceleration_x,acceleration_y;
   float start_speed_x,start_speed_y;
   unsigned char start_alpha;
   unsigned char alpha_step;
   float scale_step_x,scale_step_y,angle_step;
   int start_life;
   int current_frame;
   int total_particles;
   int generate_count,generate_time,stop_count;
   int particle_a_time;
   int action,type;
   AR_Sprite **particles;

  public:

   AR_ParticleGenerator(string filename, AR_Display *display, int total_particles, AR_Sprite *p = NULL)
    {
	 this->total_particles = total_particles;
	 particles = (AR_Sprite **)malloc(sizeof(AR_Sprite *)*total_particles);
	 for(int i=0;i<total_particles;i++)
	  {
	   if(i==0)
	    {
		 if(!p) particles[0] = new AR_Sprite(filename,display,0,0);
		 else   particles[0] = new AR_Sprite("",      display,0,0,p); // Take data from passed AR_Sprite
	    }
       else     particles[i] = new AR_Sprite("",      display,0,0,particles[0]);
	  }
	 reset();
	}

   ~AR_ParticleGenerator() { }

   void run()
    {
     for(int i=0;i<total_particles;i++)
      {
       particles[i]->run();
       if(particles[i]->getEnable())
        {
         if(particles[i]->getLife()  == 0) particles[i]->setAlpha(particles[i]->getAlpha() - alpha_step);
         if(particles[i]->getAlpha() == 0) particles[i]->setEnable(false);
         if(particles[i]->getLife()   > 0) particles[i]->setLife(particles[i]->getLife() - 1);
         particles[i]->setScaleX(particles[i]->getScaleX() + scale_step_x);
         particles[i]->setScaleY(particles[i]->getScaleY() + scale_step_y);
         particles[i]->setAngle (particles[i]->getAngle()  + angle_step);
	    }
      }
     if(action == AR::GENERATE)
      {
       if(!can_generate) { generate_count++; if(generate_count == generate_time) can_generate = true; }
       if(can_generate)
        {
		 int max;
		
         can_generate = false; generate_count = 0;
         max = particle_a_time;
	     for(int i=0;i<total_particles;i++)
          if(!particles[i]->getEnable())
           {
			if(random_enable)
			 {
			  // Random direction particle generator (useful for explosion)
	          particles[i]->setSpeedX(start_speed_x * ( ((float)AR_Util::random(100) / 10000) - ((float)AR_Util::random(100) / 10000)) );
	          particles[i]->setSpeedY(start_speed_y * ( ((float)AR_Util::random(100) / 10000) - ((float)AR_Util::random(100) / 10000)) );
		     }
            else
             {
	          particles[i]->setSpeedX(start_speed_x);
	          particles[i]->setSpeedY(start_speed_y);
			 }
	        particles[i]->setAccelerationX(acceleration_x);
	        particles[i]->setAccelerationY(acceleration_y);
	        particles[i]->setCurrentFrame(current_frame);
	        particles[i]->setX(x);
	        particles[i]->setY(y);
	        particles[i]->setEnable(true);
	        particles[i]->setAlpha(start_alpha);
	        particles[i]->setLife(start_life);
	        particles[i]->setScaleX(start_scale_x);
	        particles[i]->setScaleY(start_scale_y);
	        can_generate   = false;
	        generate_count = 0;
	        if(max==1) break; else max--;
           }

         // Generate a finite numbers of particles then stop
         if(stop_count == 1) action = AR::NONE;
         if(stop_count >= 1) stop_count--;
        }
      }
	}

   void draw() { for(int i=0;i<total_particles;i++) particles[i]->draw(); }

   void start(float x, float y, int generate_time, int stop_count = 0)
    {
	 // Start "stop_count" loops of particles generation (infinite loops if "stop_count" is 0)
	 action = AR::GENERATE;
	 this->generate_time = generate_time;
	 this->stop_count = stop_count;
	 setSource(x,y);
	}

   void stop()
    {
	 action = AR::NONE;
	}

   void reset()
    {
	 x = y = 0;
	 action          = AR::NONE;
	 can_generate    = true;
	 random_enable   = false;
	 type            = 0;
	 generate_count  = 0;
	 generate_time   = 2;
	 particle_a_time = 1;
	 stop_count      = 0;
	 current_frame   = 0;
	 start_alpha     = 255;
	 alpha_step      = 1;
	 angle_step      = 0;
	 start_life      = 10;
	 start_scale_x   = start_scale_y  = 1;
	 scale_step_x    = scale_step_y   = 0;
	 acceleration_x  = acceleration_y = 0;
     start_speed_x   = start_speed_y  = 0;
     for(int i=0;i<total_particles;i++)
      {
       particles[i]->setEnable(false);
       particles[i]->setCenterX(particles[i]->getWidth()>>1); particles[i]->setCenterY(particles[i]->getHeight()>>1);
      }
	}

   void setSource(float x, float y)
    {
	 this->x = x;
	 this->y = y;
	}

   void setStartScale(float start_scale_x, float start_scale_y)
    {
	 this->start_scale_x = start_scale_x;
	 this->start_scale_y = start_scale_y;
	}

   void setAcceleration(float acceleration_x, float acceleration_y)
    {
	 this->acceleration_x = acceleration_x;
	 this->acceleration_y = acceleration_y;
	}

   void setStartSpeed(float start_speed_x, float start_speed_y)
    {
	 this->start_speed_x = start_speed_x;
	 this->start_speed_y = start_speed_y;
	}

   void setGenerateTime(int generate_time)
    {
	 this->generate_time=generate_time;
	 generate_count = 0;
	 can_generate = true;
	}

   void setScaleStep(float scale_step_x, float scale_step_y)
    {
	 this->scale_step_x = scale_step_x;
	 this->scale_step_y = scale_step_y;
	}

   void setScrollXYP(float *scroll_x, float * scroll_y)
    {
	 for(int i=0;i<total_particles;i++)
	  {
       particles[i]->setScrollXP(scroll_x);
       particles[i]->setScrollYP(scroll_y);
	  }
	}

   void setStartLife(int start_life)             { this->start_life = start_life;           }

   void setAngleStep(float angle_step)           { this->angle_step = angle_step;           }

   void setRandomEnable(bool random_enable)      { this->random_enable = random_enable;     }

   void setCurrentFrame(int current_frame)       { this->current_frame = current_frame;     }

   void setParticleATime(int particle_a_time)    { this->particle_a_time = particle_a_time; }

   void setStartAlpha(unsigned char start_alpha) { this->start_alpha = start_alpha;         }
   
   void setAlphaStep(unsigned char alpha_step)   { this->alpha_step = alpha_step;           }

   AR_Sprite *getParticleP(int n) { return particles[n]; }

 };
