class Game : AR_Game
 {
  private:

   // Game variables
   AR_Bitmap *loader,*menu,*shop;
   AR_Font   *font_menu,*font_tiny;
   Osd    *osd;
   Stage  *stages[N_STAGES];
   Player *player;
   Hunter    *hunters   [N_HUNTERS];
   Armadillo *armadillos[N_ARMADILLOS];
   Bandit    *bandits   [N_BANDITS];
   Cruiser   *cruisers  [N_CRUISERS];
   AR_Sprite *particles,*cannons,*shields_big,*shields_little,*shields_medium;
   AR_Sprite *player_cannons;
   AR_Sprite *lights;
   AR_Sprite *cursor,*icons,*stagemap;
   AR_Sample *samples[N_SAMPLES];
   AR_Sample *sample_menu,*music_menu;
   int coins;
   int current_stage,current_wave,max_stage;
   int remaining_enemies;
   int menu_page,menu_item;
   stringstream message;

   void showDebug()
    {
	 int line;
	 stringstream buffer;

     line = 0;
	 buffer << "FPS: " << fps;
	 AR_Util::printText(font->getFont(),buffer.str(),0,font->getLineHeight() * line++);
	 AR_Util::printText(font->getFont(),string("Version: ") + AR::version(),0,font->getLineHeight() * line++);
	 buffer.str("");
	 buffer << "Angles: " << player->getCannonAngle(0) << " " << player->getCannonAngle(1);
	 AR_Util::printText(font->getFont(),buffer.str(),0,font->getLineHeight() * line++);
	 buffer.str("");
	 buffer << "Scroll X,Y: " << stages[current_stage]->getScrollX() << " " << stages[current_stage]->getScrollY();
	 AR_Util::printText(font->getFont(),buffer.str(),0,font->getLineHeight() * line++);
     buffer.str("");
	 buffer << "Stage " << current_stage << "-" << current_wave << " (" << current_stage +1 << "-" << current_wave + 1 << ") Enemies remaining: " << remaining_enemies;
	 AR_Util::printText(font->getFont(),buffer.str(),0,font->getLineHeight() * line++);

     stages[current_stage]->debug();
	 player->debug();
	 player->getShield()->debug();
	 player->getMissileLauncher()->debug();
	 for(int i=0;i<N_HUNTERS;i++)    { hunters[i]   ->debug(); hunters[i]   ->getShield()->debug(); }
	 for(int i=0;i<N_ARMADILLOS;i++) { armadillos[i]->debug(); armadillos[i]->getShield()->debug(); }
	 for(int i=0;i<N_BANDITS;i++)    { bandits[i]   ->debug(); bandits[i]   ->getShield()->debug(); }
	 for(int i=0;i<N_CRUISERS;i++)   { cruisers[i]  ->debug(); cruisers[i]  ->getShield()->debug(); }
	}

  public:

   Game(int screen_resolution, bool fullscreen) : AR_Game(screen_resolution,fullscreen) { }

   ~Game() { }

   void initGame()
    {
	 int p;

     // Loader game data
     loader = new AR_Bitmap(AR::dataPath() + "loader.png",display);
     p = 0;

     // Load stages
     AR_Util::loader(loader->getBitmap(),font->getFont(),"Loading stages ...",p,getScreenWidth());
     menu     = new AR_Bitmap(AR::dataPath()   + "menu.png",    display);
     stagemap = new AR_Sprite(AR::objectPath() + "stagemap.png",display,0,0);
     cursor   = new AR_Sprite(AR::objectPath() + "cursor.png",  display,0,0);
     icons    = new AR_Sprite(AR::objectPath() + "icons.png",   display,0,0);
     shop     = new AR_Bitmap(AR::dataPath()   + "shop.png",    display);
     osd      = new Osd(AR::dataPath()         + "osd.png",     display,icons);
     for(int i=0;i<N_STAGES;i++)
      {
       stages[i] = new Stage(AR::stagePath(i),display);
       p+=6; AR_Util::loader(loader->getBitmap(),font->getFont(),"Loading stages ...",p,getScreenWidth());
      }

     // Load musics // TODO fare AR_Music ed usarlo in classe stage....
     AR_Util::loader(loader->getBitmap(),font->getFont(),"Loading musics ...",p,getScreenWidth());
     music_menu = new AR_Sample(AR::musicPath() + "menu.ogg",audio);
     p+=14; AR_Util::loader(loader->getBitmap(),font->getFont(),"Loading musics ...",p,getScreenWidth());

     // Load sounds
     AR_Util::loader(loader->getBitmap(),font->getFont(),"Loading sounds ...",p,getScreenWidth());
     samples[0]  = new AR_Sample(AR::soundPath() + "cannon0.ogg"   ,audio);
     samples[1]  = new AR_Sample(AR::soundPath() + "cannon1.ogg"   ,audio);
     samples[2]  = new AR_Sample(AR::soundPath() + "cannon2.ogg"   ,audio);
     samples[3]  = new AR_Sample(AR::soundPath() + "cannon3.ogg"   ,audio);
     samples[4]  = new AR_Sample(AR::soundPath() + "cannon4.ogg"   ,audio);
     samples[5]  = new AR_Sample(AR::soundPath() + "cannon5.ogg"   ,audio);
     samples[6]  = new AR_Sample(AR::soundPath() + "cannon6.ogg"   ,audio);
     samples[7]  = new AR_Sample(AR::soundPath() + "metal_hit.ogg" ,audio);
     samples[8]  = new AR_Sample(AR::soundPath() + "shield_hit.ogg",audio); //N_SAMPLES è il numero massiomo di sample utilizzati
     sample_menu = new AR_Sample(AR::soundPath() + "menu.ogg"      ,audio);
     p+=6; AR_Util::loader(loader->getBitmap(),font->getFont(),"Loading sounds ...",p,getScreenWidth());

     // Load common objects, player and enemies
     AR_Util::loader(loader->getBitmap(),font->getFont(),"Loading objects ...",p,getScreenWidth());
     font_menu      = new AR_Font("data/font_menu.png",display);
     font_tiny      = new AR_Font("data/font_tiny.png",display);
     particles      = new AR_Sprite(AR::objectPath() + "particles.png",     display,0,0); // Common particles (used also for player)
     cannons        = new AR_Sprite(AR::objectPath() + "cannons.png",       display,0,0); // Common cannons
     shields_big    = new AR_Sprite(AR::objectPath() + "shields_big.png",   display,0,0); // Common big shields (used also for player)
     shields_medium = new AR_Sprite(AR::objectPath() + "shields_medium.png",display,0,0); // Common medium shields
     shields_little = new AR_Sprite(AR::objectPath() + "shields_little.png",display,0,0); // Common little shields
     lights         = new AR_Sprite(AR::objectPath() + "lights.png",        display,0,0); // Common lights (used also for player)
     player_cannons = new AR_Sprite(AR::objectPath() + "player/cannon.png", display,0,0); // Player cannons
     player         = new Player   (AR::objectPath() + "player/base.png",   display,this,particles,player_cannons,shields_big,lights,samples);
     for(int i=0;i<N_HUNTERS;i++)
      {
	   if(!i) hunters[0] = new Hunter(AR::objectPath() + "enemies/hunter.png",display,particles,cannons,shields_little,lights);
       else   hunters[i] = new Hunter("",                                     display,particles,cannons,shields_little,lights,hunters[0]);
      }
     for(int i=0;i<N_ARMADILLOS;i++)
      {
	   if(!i) armadillos[0] = new Armadillo(AR::objectPath() + "enemies/armadillo.png",display,particles,cannons,shields_little,lights);
       else   armadillos[i] = new Armadillo("",                                        display,particles,cannons,shields_little,lights,armadillos[0]);
      }
     for(int i=0;i<N_BANDITS;i++)
      {
	   if(!i) bandits[0] = new Bandit(AR::objectPath() + "enemies/bandit.png",display,particles,cannons,shields_little,lights);
       else   bandits[i] = new Bandit("",                                     display,particles,cannons,shields_little,lights,bandits[0]);
      }
     for(int i=0;i<N_CRUISERS;i++)
      {
	   if(!i) cruisers[0] = new Cruiser(AR::objectPath() + "enemies/cruiser.png",display,particles,cannons,shields_medium,lights);
       else   cruisers[i] = new Cruiser("",                                      display,particles,cannons,shields_medium,lights,cruisers[0]);
      }
     p+=8; AR_Util::loader(loader->getBitmap(),font->getFont(),"Loading objects ...",p,getScreenWidth());

     AR_Util::loader(loader->getBitmap(),font->getFont(),"Loading objects ...",p,getScreenWidth()); // Avoid fade bug
     AR_Util::wait(0.1); delete(loader);

     // Game variables and pointers init
     current_stage     = current_wave = max_stage = 0;
     remaining_enemies = 0;
     coins             = 0;
     menu_page = menu_item = 0;
     player->setScoreP(&score);
     player->setRemainingEnemiesP(&remaining_enemies);
     player->setHuntersP((Object **)hunters);
     player->setArmadillosP((Object **)armadillos);
     player->setBanditsP((Object **)bandits);
     player->setCruisersP((Object **)cruisers);
     for(int i=0;i<N_HUNTERS;i++)    { hunters[i]   ->setPlayerP(player); hunters[i]   ->setSamplesP(samples); }
     for(int i=0;i<N_ARMADILLOS;i++) { armadillos[i]->setPlayerP(player); armadillos[i]->setSamplesP(samples); }
     for(int i=0;i<N_BANDITS;i++)    { bandits[i]   ->setPlayerP(player); bandits[i]   ->setSamplesP(samples); }
     for(int i=0;i<N_CRUISERS;i++)   { cruisers[i]  ->setPlayerP(player); cruisers[i]  ->setSamplesP(samples); }
     updateStagePointers();
     cursor->startAnimation(0,6,7);
     osd->setX(0);
     osd->setY(672);
     osd->setPlayerP(player);
     osd->setScoreP(&score);
     osd->setCoinsP(&coins);
     display->fade();
    }

   void runGame()
    {
	 // Game run
	 action = AR::GAME_INTRO; timer->start();
	 while(action != AR::GAME_EXIT)
	  {
       getStartTime();
       AR_Util::clearColor(0,0,0);
       keyboard->getEvent();

       switch(action)
        {
		 case AR::GAME_INTRO:       { gameIntro();      break; }
		 case AR::GAME_MENU:        { gameMenu();       break; }
		 case AR::GAME_START:       { gameStart();      break; }
		 case AR::GAME_PROLOGUE:    { gamePrologue();   break; }
		 case AR::GAME_START_STAGE: { gameStartStage(); break; }
		 case AR::GAME_START_WAVE:  { gameStartWave();  break; }
		 case AR::GAME_END_WAVE:    { gameEndWave();    break; }
		 case AR::GAME_OVER:        { gameOver();       break; }
		 case AR::GAME_SHOP:        { gameShop();       break; }
		 case AR::GAME_EPILOGUE:    { gameEpilogue();   break; }
		 case AR::GAME_PLAY:        { gamePlay();       break; }
        }
	   drawToDisplay();
       waitFPS();
	  }
	}

   void gameIntro()
    {
     // Play game intro
     stringstream buffer;
 	 int sx,sy,fh;

     fh = font->getLineHeight();
	 sy = (display->getHeight() - fh) / 2;
	 buffer << "Valarsoft presents...";
	 sx = (display->getWidth() - font->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font->getFont(),buffer.str(),sx,sy);
	 if(generalKey() || timer->getCount() > 200)
	  {
	   action = AR::GAME_MENU; timer->stop(); display->fade();
	  }
    }

   void gameMenu()
    {
	 // Game menu
	 int mx,my,fh,max_items;

	 mx = 740; my = 540; fh = font_menu->getLineHeight();
	 menu->draw();
     cursor->setX(mx); cursor->setY(my+menu_item*fh);
     cursor->run();
     cursor->draw();
     if(!music_menu->isPlaying()) music_menu->play();
     AR_Util::printText(font_tiny->getFont(),"V." + AR::version(),display->getWidth() - font_tiny->getTextWidth("V." + AR::version()),750);
	 switch(menu_page)
	  {
	   case 0: // Main menu
	    {
		 max_items = 3;
	     AR_Util::printText(font_menu->getFont(),"MAIN MENU:       ",mx,my-2*fh);
	     AR_Util::printText(font_menu->getFont(),"  GAME SETUP   >>",mx,my+0*fh);
	     AR_Util::printText(font_menu->getFont(),"  GAME OPTIONS >>",mx,my+1*fh);
	     AR_Util::printText(font_menu->getFont(),"  GAME START     ",mx,my+2*fh);
	     AR_Util::printText(font_menu->getFont(),"  GAME EXIT      ",mx,my+3*fh);
	     if(keyboard->keyDown(key_up))    { sample_menu->play(); if(menu_item > 0)         menu_item--; }
	     if(keyboard->keyDown(key_down))  { sample_menu->play(); if(menu_item < max_items) menu_item++; }
	     if(keyboard->keyDown(key_right)) {
                                           sample_menu->play();
                                           if(menu_item == 0) { menu_page = 1; menu_item = 0; }
			                               if(menu_item == 1) { menu_page = 2; menu_item = 0; }
			                              }
	     if(generalKey())                 {
		                                   if(menu_item == 2) { action = AR::GAME_START; display->fade(); }
		                                   if(menu_item == 3) { action = AR::GAME_EXIT;  display->fade(); }
	                                      }
	     break;
	    }
	   case 1:
	    {
		 max_items = 0;
	     AR_Util::printText(font_menu->getFont(),"SETUP MENU:      ",mx,my-2*fh);
	     AR_Util::printText(font_menu->getFont(),"  BACK <<        ",mx,my+0*fh);
	     if(keyboard->keyDown(key_left)) { menu_page = 0; menu_item = 0; }
	     break;
	    }
	   case 2:
	    {
		 max_items = 0;
	     AR_Util::printText(font_menu->getFont(),"OPTIONS MENU:    ",mx,my-2*fh);
	     AR_Util::printText(font_menu->getFont(),"  BACK <<        ",mx,my+0*fh);
	     if(keyboard->keyDown(key_left)) { menu_page = 0; menu_item = 0; }
	     break;
	    }
      }
    }

   void gameStart()
    {
	 // Start new game
     current_stage = 0;
     current_wave  = 0;
     coins         = 0;
     score         = 0;
     menu_item = menu_page = 0;
     player->reset();
     for(int i=0;i<N_HUNTERS;i++)    hunters[i]   ->reset();
     for(int i=0;i<N_ARMADILLOS;i++) armadillos[i]->reset();
     for(int i=0;i<N_BANDITS;i++)    bandits[i]   ->reset();
     for(int i=0;i<N_CRUISERS;i++)   cruisers[i]  ->reset();
     updateStagePointers();
     resetScroll();
     action = AR::GAME_PROLOGUE;
    }

   void gamePrologue()
    {
     // Game prologue TODO...
     stringstream buffer;
	 int sx,sy,fh;

     fh = font_menu->getLineHeight();
	 sy = (stages[0]->getMapHeight(1) - fh) / 3;

	 buffer.str("\"... THE MOST PRECIOUS SUBSTANCE IN THE UNIVERSE");
	 sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*0);
	 buffer.str("IS THE SPICE MELANGE. THE SPICE EXTENDS LIFE.");
	 sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*1);
	 buffer.str("THE SPICE EXPANDS CONSCIOUSNESS. THE SPICE IS");
	 sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*2);
	 buffer.str("VITAL TO SPACE TRAVEL ...\"");
	 sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*3);
	 buffer.str("                                           DUNE.");
	 sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*5);

     if(generalKey())
      {
	   keyboard->clearKeys();

       action = AR::GAME_START_STAGE;
       timer->start();
       display->fade();
      }
	}

   void gameStartStage()
    {
     // New stage start
     stringstream buffer,info;
     int x,y,w;
     int max_items,basic_coins;

     basic_coins = 1500;
     if(max_stage < current_stage) max_stage = current_stage;


if(current_stage>2) { max_stage = current_stage = 0; updateStagePointers(); coins = 0; }//TODO other stages (only 2 stages for now)


     AR_Util::clearColor(69,78,91);
	 for(int i=0;i<N_STAGES;i++)
	  {
	   if(i <= max_stage) { buffer.str(stages[i]->getStageName()); info.str(""); info << "*" << i+1; }
	   else               { buffer.str("UNKNOWN WORLD"); info.str(""); }
	   w = (font_menu->getTextWidth(buffer.str())-250)/2;
	   x = 3+(i%4)*256; y = 50+(i/4)*220;
	   stagemap->setX(x); stagemap->setY(y);
	   if(i <= max_stage) stagemap->setCurrentFrame(i+1);
	   else               stagemap->setCurrentFrame(0);
	   stagemap->draw();
	   AR_Util::printText(osd->getFont()->getFont(),info.str(),x+3,y+3);
	   AR_Util::printText(font_menu->getFont(),buffer.str(),   x-w,y+190);
      }

     // At first gamemap you can select the stage to start
     if(current_stage == 0)
      {
	   max_items = max_stage;
	   AR_Util::printText(font_menu->getFont(),"SELECT STAGE TO START",3,6);
	   stagemap->setX(3+(menu_item%4)*256); stagemap->setY(50+(menu_item/4)*220);
	   stagemap->setCurrentFrame(14);
	   stagemap->draw();
       if(keyboard->keyDown(key_up))    { if(menu_item > 3)                             menu_item-=4; }
       if(keyboard->keyDown(key_down))  { if(menu_item < max_items - 3)                 menu_item+=4; }
       if(keyboard->keyDown(key_left))  { if(menu_item > 0 && menu_item%4 != 0)         menu_item--;  }
       if(keyboard->keyDown(key_right)) { if(menu_item < max_items && menu_item%4 != 3) menu_item++;  }
	  }
	 else
	  {
       // Show next stage arrow
   	   stagemap->setX(3+(current_stage%4)*256); stagemap->setY(30+timer->getCount()%30+(current_stage/4)*220);
	   stagemap->setCurrentFrame(13);
	   stagemap->draw();
	  }

     if(generalKey())
      {
	   keyboard->clearKeys();

       if(current_stage == 0)
        {
		 current_stage = menu_item;
		 coins = basic_coins * (current_stage + 1);
		 updateStagePointers();
	    }

       action = AR::GAME_START_WAVE;
       timer->start();
       display->fade();
      }
	}

   void gameStartWave()
    {
     // New wave start
     stringstream buffer;
	 int sx,sy,fh;

     fh = font_menu->getLineHeight();
	 sy = (stages[0]->getMapHeight(1) - fh) / 2;
	 buffer << "STAGE " << current_stage + 1 << " WAVE " << current_wave + 1 << " START!";
	 sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy);

     if(generalKey() || timer->getCount() > 200)
      {
	   keyboard->clearKeys();
	   timer->stop();

       // Restart player
       player->restart();

       // Restart enemies
       for(int i=0;i<N_HUNTERS;i++) // Hunters
        {
         hunters[i]->restart       (i < stages[current_stage]->getEnemiesNumber(0,current_wave));
         hunters[i]->activateShield(i < stages[current_stage]->getEnemiesNumber(0,current_wave) * stages[current_stage]->getEnemiesShield(0,current_wave));
	    }
       for(int i=0;i<N_ARMADILLOS;i++) // Armadillos
        {
         armadillos[i]->restart       (i < stages[current_stage]->getEnemiesNumber(1,current_wave));
         armadillos[i]->activateShield(i < stages[current_stage]->getEnemiesNumber(1,current_wave) * stages[current_stage]->getEnemiesShield(1,current_wave));
	    }
       for(int i=0;i<N_BANDITS;i++) // Bandits
        {
         bandits[i]->restart       (i < stages[current_stage]->getEnemiesNumber(2,current_wave));
         bandits[i]->activateShield(i < stages[current_stage]->getEnemiesNumber(2,current_wave) * stages[current_stage]->getEnemiesShield(2,current_wave));
	    }
       for(int i=0;i<N_CRUISERS;i++) // Cruisers
        {
         cruisers[i]->restart       (i < stages[current_stage]->getEnemiesNumber(3,current_wave));
         cruisers[i]->activateShield(i < stages[current_stage]->getEnemiesNumber(3,current_wave) * stages[current_stage]->getEnemiesShield(3,current_wave));
	    }
       // Restart other values
       remaining_enemies = stages[current_stage]->getTotalEnemies(current_wave); // Remaining enemies for the wave
       resetScroll();
       menu_item = menu_page = 0;
       action = AR::GAME_PLAY;
       display->fade();
      }
	}

   void gameEndWave()
    {
	 // Current stage/wave end
	 stringstream buffer;
	 float bf,bh;
	 int   p,ek;
     int   sx,sy,fh,max_items;

     // End stage animation TODO...
     max_items = 0;
     fh = font_menu->getLineHeight();
	 sy = ((stages[0]->getMapHeight(1) - fh) / 2) - fh * 7;

	 buffer << "STAGE " << current_stage + 1 << " WAVE " << current_wave + 1 << " CLEARED!";
	 sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*0);
     buffer.str("");
	 buffer << "SAVED MELANGE " << player->getCargo() << " * " << (current_stage + 1) << " = " << player->getCargo() * (current_stage + 1) << " COINS";
	 sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*2);
     bf = player->totalBulletsFired();
     bh = player->getBulletsHit();
     p  = (int)(bh / bf * 100);
     ek = stages[current_stage]->getTotalEnemies(current_wave); // Enemies killed
     buffer.str("");
     buffer << "ENEMIES KILLED " << ek << " BULLETS FIRED " << bf << " BULLETS HIT " << bh << " PRECISION " << p << "%";
     sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*3);
	 if(p<50)
	  {
       buffer.str("");
	   buffer << "NO BONUS!";
	   sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	   AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*4);
      }
     if(p>=50)
	  {
       buffer.str("");
	   buffer << "SCORE BONUS: " << ek << " * " << p << " * 2 = " << 2*ek*p;
	   sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	   AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*4);
       buffer.str("");
	   buffer << "COINS BONUS: " << ek << " * " << p/10 << " = " << ek*(p/10);
	   sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	   AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*5);
      }
     if(p==100)
	  {
       buffer.str("");
	   buffer << "EXTRA SCORE BONUS 500";
	   sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	   AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*6);
       buffer.str("");
	   buffer << "EXTRA COINS BONUS 250";
	   sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	   AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*7);
      }

     // Continue/Shop menu
     buffer.str("");
     buffer << "  CONTINUE TO NEXT WAVE";
	 sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*9);
	 if(current_wave + 1 != N_WAVES || current_stage + 1 != N_STAGES) // In the last wave of the last stage you can not go to shop
	  {
	   max_items = 1;
       buffer.str("");
       buffer << "  GO TO SHOP!";
	   AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*10);
       if(keyboard->keyDown(key_up))   { if(menu_item > 0)         menu_item--; }
       if(keyboard->keyDown(key_down)) { if(menu_item < max_items) menu_item++; }
      }
     cursor->setX(sx); cursor->setY(sy+fh*(menu_item+9));
     cursor->run();
     cursor->draw();
     if(generalKey())
      {
	   // Get coins and bonus
       coins += player->getCargo() * (current_stage + 1);
       if(p>=50)  { score += 2*ek*p; coins += ek*(p/10); }
       if(p==100) { score += 500;    coins += 250; }
       if(menu_item == 0) // Next wave
        {
         current_wave++;
         if(current_wave  == N_WAVES)  { current_stage++; current_wave = 0; updateStagePointers(); action = AR::GAME_START_STAGE; timer->start(); }
         else                          { action = AR::GAME_START_WAVE; timer->start(); }
         if(current_stage == N_STAGES) { action = AR::GAME_EPILOGUE;   timer->start(); }
         display->fade();
        }
       if(menu_item == 1) // Go to shop
        {
         current_wave++;
         if(current_wave  == N_WAVES)  { current_stage++; current_wave = 0; updateStagePointers(); action = AR::GAME_SHOP; menu_page = menu_item = 0; }
         else                          { action = AR::GAME_SHOP;     timer->start(); menu_page = menu_item = 0; }
         if(current_stage == N_STAGES) { action = AR::GAME_EPILOGUE; timer->start(); }
         display->fade();
		}
	  }
	}

   void gameShop()
    {
     // Shop
     stringstream buffer;
     int max_items,max_inventory_items,inventory_item,fh,j;
     int    *inv;
     Weapon **wea;
     Item   **ite;

     max_items = 16;
     max_inventory_items = player->countInventory() * 3;
     fh = font_tiny->getLineHeight();
	 shop->draw();

     // Weapons
	 wea = player->getWeaponsP();
	 for(int i=0; i<N_WEAPONS;i++)
	  {
	   j = i;
	   icons->setX(4+(j%4)*256); icons->setY(41+j/4*122); icons->setCurrentFrame(wea[i]->getIconFrame()); icons->draw();
	   buffer.str(""); buffer << wea[i]->getName();
	   AR_Util::printText(osd->getFont()->getFont(),buffer.str(), 3+(j%4)*256, 22+j/4*122);
	   buffer.str(""); buffer << "FIRERATE " << wea[i]->getFirerate() << "/MINUTE";
	   AR_Util::printText(font_tiny->getFont(),buffer.str(),     47+(j%4)*256, 41+j/4*122+fh*0);
	   buffer.str(""); buffer << "RANGE " << wea[i]->getPower() * 100 << " METERS";
	   AR_Util::printText(font_tiny->getFont(),buffer.str(),     47+(j%4)*256, 41+j/4*122+fh*1);
	   buffer.str(""); buffer << "DAMAGE " << wea[i]->getBulletDamage();
	   AR_Util::printText(font_tiny->getFont(),buffer.str(),     47+(j%4)*256, 41+j/4*122+fh*2);
	   buffer.str(""); buffer << wea[i]->getDescription(0);
	   AR_Util::printText(font_tiny->getFont(),buffer.str(),      3+(j%4)*256, 41+j/4*122+fh*3);
	   buffer.str(""); buffer << wea[i]->getDescription(1);
	   AR_Util::printText(font_tiny->getFont(),buffer.str(),      3+(j%4)*256, 41+j/4*122+fh*4);
	   buffer.str(""); buffer << "  BUY FOR " << wea[i]->getCost() << " COINS";
	   AR_Util::printText(font_menu->getFont(),buffer.str(),      3+(j%4)*256, 41+j/4*122+fh*5);
      }
     
     // Items
     ite = player->getItemsP();
	 for(int i=0; i<N_ITEMS;i++)
	  {
	   j = i + N_WEAPONS;
	   icons->setX(4+(j%4)*256); icons->setY(41+j/4*122); icons->setCurrentFrame(ite[i]->getIconFrame()); icons->draw();
	   buffer.str(""); buffer << ite[i]->getName();
	   AR_Util::printText(osd->getFont()->getFont(),buffer.str(), 3+(j%4)*256, 22+j/4*122);
	   buffer.str(""); buffer << ite[i]->getDescription(0);
	   AR_Util::printText(font_tiny->getFont(),buffer.str(),     47+(j%4)*256, 41+j/4*122+fh*0);
	   buffer.str(""); buffer << ite[i]->getDescription(1) << ite[i]->getPercentageString();
	   AR_Util::printText(font_tiny->getFont(),buffer.str(),     47+(j%4)*256, 41+j/4*122+fh*1);
	   buffer.str(""); buffer << ite[i]->getDescription(2);
	   AR_Util::printText(font_tiny->getFont(),buffer.str(),     47+(j%4)*256, 41+j/4*122+fh*2);
	   buffer.str(""); buffer << ite[i]->getDescription(9);
	   AR_Util::printText(font_tiny->getFont(),buffer.str(),      3+(j%4)*256, 41+j/4*122+fh*3);
	   buffer.str(""); buffer << ite[i]->getDescription(10);
	   AR_Util::printText(font_tiny->getFont(),buffer.str(),      3+(j%4)*256, 41+j/4*122+fh*4);
	   if(ite[i]->getClicks() <= ite[i]->getMaxClicks()) { buffer.str(""); buffer << "  BUY FOR " << ite[i]->getCost() << " COINS"; }
	   else                                              { buffer.str(""); buffer << "  ALL SOLDED!"; }
	   AR_Util::printText(font_menu->getFont(),buffer.str(),      3+(j%4)*256, 41+j/4*122+fh*5);
      }

     // Inventory
     j = 0;
     fh = font_menu->getLineHeight();
     inv = player->getInventoryP();
   	 buffer.str(""); buffer << "INVENTORY:";
	 AR_Util::printText(osd->getFont()->getFont(),buffer.str(),257,507);
	 for(int i=0; i<N_INVENTORY;i++)
	  {
	   if(inv[i] != -1)
	    {
	     icons->setX(260+190*j); icons->setY(546); icons->setCurrentFrame(wea[inv[j]]->getIconFrame()); icons->draw();
	     buffer.str(""); buffer << wea[inv[j]]->getName();
	     AR_Util::printText(osd->getFont()->getFont(),buffer.str(), 259+190*j, 527);
	     buffer.str(""); buffer << "  EQUIP CANNON 1";
	     AR_Util::printText(font_menu->getFont(),buffer.str(),      259+190*j, 590+fh*0);
	     buffer.str(""); buffer << "  EQUIP CANNON 2";
	     AR_Util::printText(font_menu->getFont(),buffer.str(),      259+190*j, 590+fh*1);
	     buffer.str(""); buffer << "  SELL FOR " << wea[inv[j]]->getSell();
	     AR_Util::printText(font_menu->getFont(),buffer.str(),      259+190*j, 590+fh*2);
	     j++;
	    }
      }
     if(j==0) {
               buffer.str(""); buffer << "EMPTY!";
	           AR_Util::printText(osd->getFont()->getFont(),buffer.str(),259,527);
	          }

	 // Exit
	 buffer.str(""); buffer << "  EXIT SHOP";
	 AR_Util::printText(font_menu->getFont(),buffer.str(),3,639);
	 osd->draw();

     // Message
	 if(!timer->isStarted()) message.str("WELCOME TO FAR FAR SHOP!");
	 if(timer->getCount() > 180) timer->stop();
	 AR_Util::printText(osd->getFont()->getFont(),message.str(),2,1);

	 // Cursor
     if(menu_item <= max_items)
      {
	   if(keyboard->keyDown(key_down)  && menu_item == max_items) { menu_item = max_items + max_inventory_items + 1; keyboard->clearKeys(); }                                      // Go to exit
	   if(keyboard->keyDown(key_right) && menu_item == max_items) { if(max_inventory_items > 0) { menu_item = max_items + 1; keyboard->clearKeys(); } }                            // Go to inventory
	   if(keyboard->keyDown(key_down) && menu_item > max_items - 4 && menu_item < max_items) { if(max_inventory_items > 0) { menu_item = max_items + 1; keyboard->clearKeys(); } } // Go to inventory
       if(keyboard->keyDown(key_up))    { if(menu_item > 3)                             menu_item-=4; }
       if(keyboard->keyDown(key_down))  { if(menu_item < max_items - 3)                 menu_item+=4; }
       if(keyboard->keyDown(key_left))  { if(menu_item > 0 && menu_item%4 != 0)         menu_item--;  }
       if(keyboard->keyDown(key_right)) { if(menu_item < max_items && menu_item%4 != 3) menu_item++;  }
	   cursor->setX(4+(menu_item%4)*256); cursor->setY(116+menu_item/4*122);
      }
     if(menu_item > max_items && menu_item <= max_items + max_inventory_items)
      {
	   inventory_item = menu_item - 17;
	   if(keyboard->keyDown(key_left) && inventory_item < 3)  { menu_item = max_items + max_inventory_items + 1; keyboard->clearKeys(); } // Go out from inventary (to left)
       if(keyboard->keyDown(key_up) && inventory_item%3 == 0) { menu_item = max_items - 3 + inventory_item/4; keyboard->clearKeys(); }    // Go out from inventory (to up)
	   if(keyboard->keyDown(key_up))    { if(inventory_item > 0 && inventory_item%3 != 0)                   menu_item--;  }
       if(keyboard->keyDown(key_down))  { if(inventory_item < max_inventory_items && inventory_item%3 != 2) menu_item++;  }
       if(keyboard->keyDown(key_left))  { if(inventory_item > 2)                                            menu_item-=3; }
       if(keyboard->keyDown(key_right)) { if(inventory_item < max_inventory_items - 3)                      menu_item+=3; }     
	   cursor->setX(260+inventory_item/3*190); cursor->setY(590+(inventory_item%3)*fh);
      }
     if(menu_item == max_items + max_inventory_items + 1)
      {
       if(keyboard->keyDown(key_up)) menu_item = 16;
       if(keyboard->keyDown(key_right) && max_inventory_items > 0) menu_item = 17;
       cursor->setX(4); cursor->setY(639);
      }
     cursor->run();
     cursor->draw();

     if(generalKey())
      {
       // Weapons
       if(menu_item < 7)
        {
         if(coins >= wea[menu_item]->getCost())
          {
           if(player->addToInventory(menu_item))
            {
             coins -= wea[menu_item]->getCost();
             message.str("ITEM STORED IN THE INVENTORY!"); timer->start();
            }
           else { message.str("INVENTORY FULL, YOU CAN NOT BUY THIS ITEM!"); timer->start(); }
          }
         else { message.str(AR::errNoCoins()); timer->start(); }
        }

       // Items
       if(menu_item - 7 == AR::ITEM_RADAR)
        {
		 if(coins >= ite[menu_item -7]->getCost())
		  {
		   if(ite[menu_item -7]->getClicks() == 0)
		    {
			 // Activate radar
		     player->setRadarEnable(true);
		     coins -= ite[menu_item -7]->getCost();
		     ite[menu_item -7]->click();
		    }
		   else
		    {
			 // Increase radar range
			 if(ite[menu_item -7]->getClicks() <= ite[menu_item -7]->getMaxClicks())
			  {
               player->incRadarRange();
               coins -= ite[menu_item -7]->getCost();
               ite[menu_item -7]->click();
		      }
			}
		  }
		 else { message.str(AR::errNoCoins()); timer->start(); }
        }

       if(menu_item - 7 == AR::ITEM_SHIELD)
        {
		 if(coins >= ite[menu_item -7]->getCost())
		  {
		   if(ite[menu_item -7]->getClicks() == 0)
		    {
			 // Activate shields
		     player->activateShield();
		     player->setShieldEnableRecharge(true);
		     player->setShieldRechargeStep(display->getMaxFPS() * 60);
		     coins -= ite[menu_item -7]->getCost();
		     ite[menu_item -7]->click();
		    }
		   else
		    {
			 // Increase shield power and shield recharge
			 if(ite[menu_item -7]->getClicks() <= ite[menu_item -7]->getMaxClicks())
			  {
               player->activateShield(false);
			   player->setShieldMaxHealth(player->getShieldMaxHealth() + 20);
               player->setShieldHealth(player->getShieldMaxHealth());
               player->setShieldEnableRecharge(true);
               player->setShieldRechargeStep((display->getMaxFPS() * 60) / (ite[menu_item -7]->getClicks() + 1));
               coins -= ite[menu_item -7]->getCost();
               ite[menu_item -7]->click();
		      }
			}
		  }
		 else { message.str(AR::errNoCoins()); timer->start(); }
		}

       if(menu_item - 7 == AR::ITEM_HULL)
        {
		 if(coins >= ite[menu_item -7]->getCost())
		  {
           // Increase hull armour
		   if(ite[menu_item -7]->getClicks() <= ite[menu_item -7]->getMaxClicks())
			{
			 player->setMaxHealth(player->getMaxHealth() + 50);
             player->setHealth(player->getMaxHealth());
             coins -= ite[menu_item -7]->getCost();
             ite[menu_item -7]->click();
		    }
		  }
		 else { message.str(AR::errNoCoins()); timer->start(); }
		}

       if(menu_item - 7 == AR::ITEM_TARGETTING)
        {
		 if(coins >= ite[menu_item -7]->getCost())
		  {
		   if(ite[menu_item -7]->getClicks() == 0)
		    {
			 // Activate targetting
		     player->getCannon(0)->setEnableTargetting(true);
		     player->getCannon(1)->setEnableTargetting(true);
		     coins -= ite[menu_item -7]->getCost();
		     ite[menu_item -7]->click();
		    }
		   else
		    {
			 // Increase targetting range
			 if(ite[menu_item -7]->getClicks() <= ite[menu_item -7]->getMaxClicks())
			  {
               player->getCannon(0)->incTargettingRange();
               player->getCannon(1)->incTargettingRange();
               coins -= ite[menu_item -7]->getCost();
               ite[menu_item -7]->click();
		      }
			}
		  }
		 else { message.str(AR::errNoCoins()); timer->start(); }
		}

       if(menu_item - 7 == AR::ITEM_CARGO)
        {
		 if(coins >= ite[menu_item -7]->getCost())
		  {
		   // Increase base store capacity and decrease cargo decay rate
		   if(ite[menu_item -7]->getClicks() <= ite[menu_item -7]->getMaxClicks())
		    {
		     player->setMaxCargo(player->getMaxCargo() + 80);
		     player->setDecayStep(5 + ite[menu_item -7]->getClicks());
		     coins -= ite[menu_item -7]->getCost();
		     ite[menu_item -7]->click();
		    }
		  }
		 else { message.str(AR::errNoCoins()); timer->start(); }
		}

       if(menu_item - 7 == AR::ITEM_ENEMIES_SCAN)
        {
		 if(coins >= ite[menu_item -7]->getCost())
		  {
		   // Enable enemies scans
		   if(ite[menu_item -7]->getClicks() <= ite[menu_item -7]->getMaxClicks())
		    {
		     if(ite[menu_item -7]->getClicks() == 0) player->setScanEnemiesRemaining(true);
		     if(ite[menu_item -7]->getClicks() == 1) { player->setScanEnemiesHealth(true); ite[menu_item -7]->setDescription(4,"SHIELDS"); }
		     if(ite[menu_item -7]->getClicks() == 2) player->setScanEnemiesShields(true);
		     coins -= ite[menu_item -7]->getCost();
		     ite[menu_item -7]->click();
		    }
		  }
		 else { message.str(AR::errNoCoins()); timer->start(); }
		}

       if(menu_item - 7 == AR::ITEM_REPAIR_HULL)
        {
		 if(coins >= ite[menu_item -7]->getCost())
		  {
           // Hull armour +100
           if(player->getHealth() < player->getMaxHealth())
            {
             player->setHealth(player->getHealth()+100);
		     if(player->getHealth() > player->getMaxHealth()) player->setHealth(player->getMaxHealth());
		     coins -= ite[menu_item -7]->getCost();
		    }
		   else { message.str("HULL IS NOT DAMAGED!"); timer->start(); }
		  }
		 else { message.str(AR::errNoCoins()); timer->start(); }
		}

       if(menu_item - 7 == AR::ITEM_REPAIR_SHIELD)
        {
		 if(coins >= ite[menu_item -7]->getCost())
		  {
           // Shield power +80
           if(player->isShieldEnable() || player->getShieldDown())
            {
             if(player->getShieldHealth() < player->getShieldMaxHealth())
              {
			   player->activateShield(false);
               player->setShieldHealth(player->getShieldHealth()+80);
		       if(player->getShieldHealth() > player->getShieldMaxHealth()) player->setShieldHealth(player->getShieldMaxHealth());
		       coins -= ite[menu_item -7]->getCost();
		      }
		     else { message.str("MAXIMUM SHIELDS POWER!"); timer->start(); }
		    }
           else { message.str("SHIELDS DISABLED, ENABLE SHIELDS FIRST!"); timer->start(); }
		  }
		 else { message.str(AR::errNoCoins()); timer->start(); }
		}

       if(menu_item - 7 == AR::ITEM_CANNON_ROTATE)
        {
		 if(coins >= ite[menu_item -7]->getCost())
		  {
           // Increase cannons rotation
		   if(ite[menu_item -7]->getClicks() <= ite[menu_item -7]->getMaxClicks())
			{
             player->getCannon(0)->setRotationSpeed(player->getCannon(0)->getRotationSpeed()+0.08);
             player->getCannon(1)->setRotationSpeed(player->getCannon(1)->getRotationSpeed()+0.08);
             coins -= ite[menu_item -7]->getCost();
             ite[menu_item -7]->click();
		    }
		  }
		 else { message.str(AR::errNoCoins()); timer->start(); }
		}

       if(menu_item - 7 == AR::ITEM_MISSILE)
        {
		 if(coins >= ite[menu_item -7]->getCost())
		  {
		   if(ite[menu_item -7]->getClicks() == 0)
		    {
			 // Activate missiles
		     player->getMissileLauncher()->setWeapon(AR::CANNON_GENERIC);
		     coins -= ite[menu_item -7]->getCost();
		     ite[menu_item -7]->click();
		    }
		   else
		    {
			 // Increase missiles recharge speed
			 if(ite[menu_item -7]->getClicks() <= ite[menu_item -7]->getMaxClicks())
			  {
               player->getMissileLauncher()->setReloadTime(player->getMissileLauncher()->getReloadTime() - 100);
               player->getMissileLauncher()->setReloadCount(player->getMissileLauncher()->getReloadTime());
               coins -= ite[menu_item -7]->getCost();
               ite[menu_item -7]->click();
		      }
			}
		  }
		 else { message.str(AR::errNoCoins()); timer->start(); }
		}

       // Inventory
       if(menu_item > max_items && menu_item < max_items + max_inventory_items + 1)
        {
		 if(inventory_item%3 == 0) // Equip cannon 0
		  {
           int t;

		   t = player->getCurrentWeapon(0);
		   player->changeWeapon(inv[inventory_item/3],0);
		   player->removeFromInventory(inventory_item/3);
		   if(t != AR::NO_CANNON) player->addToInventory(t);
		   menu_item = max_items + 1;
	      }
		 if(inventory_item%3 == 1) // Equip cannon 1
		  {
           int t;

		   t = player->getCurrentWeapon(1);
		   player->changeWeapon(inv[inventory_item/3],1);
		   player->removeFromInventory(inventory_item/3);
		   if(t != AR::NO_CANNON) player->addToInventory(t);
		   menu_item = max_items + 1;
	      }
		 if(inventory_item%3 == 2) // Sell
		  {
		   coins += wea[inv[inventory_item/3]]->getSell();
		   player->removeFromInventory(inventory_item/3);
		   menu_item = max_items + 1;
	      }
		}
       if(menu_item == max_items + max_inventory_items + 1) // Exit from shop
        {
         if(current_wave == 0) action = AR::GAME_START_STAGE;
         else                  action = AR::GAME_START_WAVE;
         menu_item = 0;
         timer->start();
         display->fade();
	    }
      }
	}

   void gameOver()
    {
     // Game over
     stringstream buffer;
	 int sx,sy,fh;

     fh = font_menu->getLineHeight();
	 sy = (stages[0]->getMapHeight(1) - fh) / 2;
	 buffer << "GAME OVER";
	 sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy);

     if(generalKey() || timer->getCount() > 200)
      {
	   timer->stop();
       action = AR::GAME_MENU;
       display->fade();
      }
	}

   void gameEpilogue()
    {
     // Game epilogue
     stringstream buffer;
	 int sx,sy,fh;

     fh = font_menu->getLineHeight();
	 sy = (stages[0]->getMapHeight(1) - fh) / 2 - fh * 4;
	 buffer << "CONGRATULATION!!!";
	 sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*0);
	 buffer.str("");
	 buffer << "SCORE " << score;
	 sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*2);
	 buffer.str("");
	 buffer << "FINAL BONUS " << coins << " * 10 = " << coins * 10;
	 sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*3);
	 buffer.str("");
	 buffer << "FINAL SCORE " << score + coins * 10;
	 sx = (display->getWidth() - font_menu->getTextWidth(buffer.str())) / 2;
	 AR_Util::printText(font_menu->getFont(),buffer.str(),sx,sy+fh*4);

     if(generalKey() || timer->getCount() > 500)
      {
	   score += coins*10;
	   timer->stop();
       action = AR::GAME_OVER;
       timer->start();
       display->fade();
      }
	}

   void gamePlay()
    {
     // Game scroll    
     stages[current_stage]->setScrollX(((stages[current_stage]->getMapWidth(1) - display->getWidth()) / 2) * cos(AR_Util::radians(360 - player->getCannonAngle(0))) + ((stages[current_stage]->getMapWidth(1) - display->getWidth()) / 2));

     // Game run
	 player->run();
	 for(int i=0;i<N_HUNTERS;i++)    hunters[i]   ->run();
	 for(int i=0;i<N_ARMADILLOS;i++) armadillos[i]->run();
	 for(int i=0;i<N_BANDITS;i++)    bandits[i]   ->run();
	 for(int i=0;i<N_CRUISERS;i++)   cruisers[i]  ->run();
	 stages[current_stage]->run();

     // Game draw stage and all objects
	 stages[current_stage]->draw();
	 stages[current_stage]->drawLayer0();
	 stages[current_stage]->drawLayer1();
     player->draw();
     for(int i=0;i<N_HUNTERS;i++)    hunters[i]   ->draw();
     for(int i=0;i<N_ARMADILLOS;i++) armadillos[i]->draw();
     for(int i=0;i<N_BANDITS;i++)    bandits[i]   ->draw();
     for(int i=0;i<N_CRUISERS;i++)   cruisers[i]  ->draw();
     stages[current_stage]->drawLayerH();
     osd->draw();

     // Show debug
     if(show_debug) showDebug();

	 // Game action
	 if(keyboard->keyDown(key_esc))                { action = AR::GAME_MENU; display->fade(); }
	 if(keyboard->keyDown(key_debug))              { toggleDebug(); }
	 if(debug) // Debug cheat keys
	  {
       if(keyboard->keyDown(ALLEGRO_KEY_F1)) stages[current_stage]->setEnableLayer(0,!stages[current_stage]->getEnableLayer(0));
       if(keyboard->keyDown(ALLEGRO_KEY_F2)) stages[current_stage]->setEnableLayer(1,!stages[current_stage]->getEnableLayer(1));
       if(keyboard->keyDown(ALLEGRO_KEY_F3)) stages[current_stage]->setEnableLayer(2,!stages[current_stage]->getEnableLayer(2));
	   if(keyboard->keyDown(ALLEGRO_KEY_F5)) coins = 100000;
	   if(keyboard->keyDown(ALLEGRO_KEY_F6)) action = AR::GAME_END_WAVE;
	  }
     if(!player->getLife())                        { action = AR::GAME_OVER; timer->start(); display->fade(); }
     if(!remaining_enemies && player->getHealth()) { action = AR::GAME_END_WAVE; display->fade(); }
    }

   void updateStagePointers()
    {
	 osd->setStageP(stages[current_stage]);
     player->setScrollXYP(stages[current_stage]->getScrollXP(),stages[current_stage]->getScrollYP());
     for(int i=0;i<N_HUNTERS;i++)    hunters[i]   ->setScrollXYP(stages[current_stage]->getScrollXP(),stages[current_stage]->getScrollYP());
     for(int i=0;i<N_ARMADILLOS;i++) armadillos[i]->setScrollXYP(stages[current_stage]->getScrollXP(),stages[current_stage]->getScrollYP());
     for(int i=0;i<N_BANDITS;i++)    bandits[i]   ->setScrollXYP(stages[current_stage]->getScrollXP(),stages[current_stage]->getScrollYP());
     for(int i=0;i<N_CRUISERS;i++)   cruisers[i]  ->setScrollXYP(stages[current_stage]->getScrollXP(),stages[current_stage]->getScrollYP());
    }
   
   void resetScroll()
    {
     stages[current_stage]->setScrollX((stages[current_stage]->getMapWidth(1) - display->getWidth()) / 2);//400
     stages[current_stage]->setScrollY(0);
	}

   bool generalKey()
    {
	 // Check some keys down
	 if(keyboard->keyDown(key_fire1))  return true;
	 if(keyboard->keyDown(key_fire2))  return true;
	 if(keyboard->keyDown(key_fire3))  return true;
	 if(keyboard->keyDown(key_select)) return true;
	 if(keyboard->keyDown(key_space))  return true;
	 return false;
	}

 };
